#include <tchar.h>
#include "AviReader.h"

C_AviReader::C_AviReader() {
	m_pGF = NULL;
	m_nWidth = 0;
	m_nHeight = 0;
	m_nRate = 0;
	m_nFrames = 0;
	m_nFramesCounter = 0;

	AVIFileExit();
	AVIFileInit();
}

C_AviReader::~C_AviReader() {
	if(m_pGF)
		AVIStreamGetFrameClose(m_pGF);
	AVIFileExit();
}

BOOL C_AviReader::Open(TCHAR *szFileName) {
	PAVIFILE pAF;
	PAVISTREAM pAS;
	AVIFILEINFO AFI;     
	AVISTREAMINFO ASI;

	Close();

	try {
		m_nFramesCounter = 0;
		if(AVIFileOpen(&pAF, szFileName, OF_READ, 0L))
			throw _T("Error in AVIFileOpen");
		if(AVIFileInfo(pAF, &AFI, sizeof(AVIFILEINFO)))
			throw _T("Error in AVIFileInfo");
		if(AVIFileGetStream(pAF, &pAS, streamtypeVIDEO, 0))
			throw _T("Error in AVIFileGetStream");

		AVIFileRelease(pAF);

		if(AVIStreamInfo(pAS, &ASI, sizeof(AVISTREAMINFO)))
			throw _T("Error in AVIStreamInfo");
		if((m_pGF = AVIStreamGetFrameOpen(pAS, 0)) == NULL)
			throw _T("Error in AVIStreamGetFrameOpen");
		AVIStreamRelease(pAS);

		LPBITMAPINFO pBMPI;
		if((pBMPI = reinterpret_cast<LPBITMAPINFO>(AVIStreamGetFrame(m_pGF, 0))) == NULL)
			throw _T("Error in AVIStreamGetFrame");
		if(pBMPI->bmiHeader.biBitCount != 24)
			throw _T("Needed 24 bits per pixel AVI");

		m_nWidth = pBMPI->bmiHeader.biWidth;
		m_nHeight = pBMPI->bmiHeader.biHeight;
		m_nRate = ASI.dwRate;
		if(ASI.dwScale)
			m_nRate /= ASI.dwScale;
		m_nFrames = AFI.dwLength;
	}
	catch(TCHAR *szMes) {
		Close();
		MessageBox(NULL, szMes, _T(""), MB_OK);
		return FALSE;
	}

	return TRUE;
}

BOOL C_AviReader::Close() {
	if(m_pGF) {
		AVIStreamGetFrameClose(m_pGF);
		m_pGF = NULL;
	}

	return TRUE;
}

BOOL C_AviReader::GetFrame(BYTE *pbFrame) {
	if(!m_pGF || !pbFrame)
		return FALSE;

	LPBITMAPINFO pBMPI = reinterpret_cast<LPBITMAPINFO>(AVIStreamGetFrame(m_pGF, m_nFramesCounter));
	if(!pBMPI)
		return FALSE;

	PBYTE pbSrc = (PBYTE)pBMPI + sizeof(BITMAPINFOHEADER);
	PBYTE pbCur = pbFrame + m_nWidth * (m_nHeight - 1);

	for(int i=0; i<m_nHeight; i++) {
		for(int j=0, k=0; j<m_nWidth; j++, k+=3)
			pbCur[j] = static_cast<BYTE>((pbSrc[k] + pbSrc[k + 1] + pbSrc[k + 2]) / 3);
		pbCur -= m_nWidth;
		pbSrc += m_nWidth * 3;
	}
	if(++m_nFramesCounter >= m_nFrames)
		m_nFramesCounter = 0;
	return TRUE;
}

